import { relative, resolve } from 'node:path/posix';
import config from './config.js';
import dialects from './dialects.js';
import { fail } from './fail.js';
import pkg from './package.js';
import polyfills from './polyfills.js';
import { resolveExport } from './resolve-export.js';
const getImpTarget = (s) => {
    if (s === undefined)
        return undefined;
    if (typeof s === 'string') {
        const imp = s.endsWith('.cts') ? undefined : s;
        return !imp || !imp.startsWith('./src/')
            ? imp
            : dialects.includes('esm')
                ? `./dist/esm/${relative(resolve('./src'), resolve(imp)).replace(/\.(m?)tsx?$/, '.$1js')}`
                : undefined;
    }
    if (s && typeof s === 'object') {
        return resolveExport(s, 'import');
    }
};
const getReqTarget = (s, polyfills) => {
    if (s === undefined)
        return undefined;
    if (typeof s === 'string') {
        const req = s.endsWith('.mts') ? undefined : s;
        return !req || !req.startsWith('./src/')
            ? req
            : dialects.includes('commonjs')
                ? `./dist/commonjs/${relative(resolve('./src'), resolve(polyfills.get(req) || req)).replace(/\.(c?)tsx?$/, '.$1js')}`
                : undefined;
    }
    if (s && typeof s === 'object') {
        return getReqTarget(resolveExport(s, 'require'), polyfills);
    }
};
const getExports = (c, polyfills) => {
    if (!c.exports) {
        fail('no exports on tshy config (is there code in ./src?)');
        process.exit(1);
    }
    const e = {};
    for (const [sub, s] of Object.entries(c.exports)) {
        const impTarget = getImpTarget(s);
        const reqTarget = getReqTarget(s, polyfills);
        // external export, not built by us
        if (typeof s !== 'string' || !s.startsWith('./src/')) {
            // already been validated, just accept as-is
            e[sub] = s;
            continue;
        }
        if (!impTarget && !reqTarget)
            continue;
        const exp = (e[sub] = {});
        if (impTarget) {
            exp.import = {
                types: impTarget.replace(/\.(m?)js$/, '.d.$1ts'),
                default: impTarget,
            };
        }
        if (reqTarget) {
            exp.require = {
                types: reqTarget.replace(/\.(c?)js$/, '.d.$1ts'),
                default: reqTarget,
            };
        }
    }
    return e;
};
// These are all defined by exports, so it's just confusing otherwise
delete pkg.module;
delete pkg.main;
delete pkg.types;
export default pkg.exports = getExports(config, polyfills);
//# sourceMappingURL=exports.js.map